@@ -682,6 +682,7 @@ std::unique_ptr< std::vector<double> > forestry::predict(
 
 std::vector<double> forestry::predictOOB(
     std::vector< std::vector<double> >* xNew,
+    arma::Mat<double>* weightMatrix,
     bool doubleOOB
 ) {
 
@@ -733,7 +734,8 @@ std::vector<double> forestry::predictOOB(
                       getOOBhonest(),
                       doubleOOB,
                       getMinNodeSizeToSplitAvg(),
-                      xNew
+                      xNew,
+                      weightMatrix
                   );
     #if DOPARELLEL
                   std::lock_guard<std::mutex> lock(threadLock);
@@ -770,6 +772,12 @@ std::vector<double> forestry::predictOOB(
       OOB_MSE +=
         pow(trueValue - outputOOBPrediction[j] / outputOOBCount[j], 2);
       outputOOBPrediction[j] = outputOOBPrediction[j] / outputOOBCount[j];
+      //Also divide the weightMatrix
+      if (weightMatrix) {
+        for (size_t i = 0; i < numObservations; i++) {
+          (*weightMatrix)(j,i) = (*weightMatrix)(j,i) / outputOOBCount[j];
+        }
+      }
     } else {
       outputOOBPrediction[j] = std::numeric_limits<double>::quiet_NaN();
     }
@@ -944,7 +952,8 @@ void forestry::calculateOOBError(
               getOOBhonest(),
               doubleOOB,
               getMinNodeSizeToSplitAvg(),
-              nullptr
+              nullptr,
+              NULL
             );
 
             #if DOPARELLEL
